@@ -38,7 +38,7 @@ int hdf5_global_verbosity;
 
 static herr_t
 ref_string (hid_t sid, hid_t did, H5T_cdata_t *cdata,
-	    size_t count, size_t stride, size_t bkg_stride,
+	    hsize_t count, size_t stride, size_t bkg_stride,
 	    void *buf, void *bkg,
 	    hid_t dset_xfer_plid)
 {
@@ -64,7 +64,7 @@ ref_string (hid_t sid, hid_t did, H5T_cdata_t *cdata,
 
 static herr_t
 string_ref (hid_t sid, hid_t did, H5T_cdata_t *cdata,
-	    size_t count, size_t stride, size_t bkg_stride,
+	    hsize_t count, size_t stride, size_t bkg_stride,
 	    void *buf, void *bkg,
 	    hid_t dset_xfer_plid)
 {
@@ -75,6 +75,7 @@ string_ref (hid_t sid, hid_t did, H5T_cdata_t *cdata,
       size_t i;
 
       memcpy (srcbuf, buf, sizeof (srcbuf));
+      srcbuf[sizeof (srcbuf) - 1] = '\0';
       for (i = 0; i < count;i ++)
 	{
 	  ((SEXPREC **)buf)[i] = mkChar (srcptr);
@@ -803,7 +804,7 @@ do_hdf5cleanup (SEXP args)
   fid = (hid_t) INTEGER (CAR (args))[0];
 
   H5Tunregister (H5T_PERS_SOFT, STRING2REF_CONV, -1, -1, string_ref);
-  H5Tunregister (H5T_PERS_SOFT, STRING2REF_CONV, -1, -1, ref_string);
+  H5Tunregister (H5T_PERS_SOFT, REF2STRING_CONV, -1, -1, ref_string);
   
   if (H5Fclose (fid) < 0)
     errorcall (call, "unable to close HDF file");
@@ -1355,7 +1356,7 @@ hdf5_process_object (hid_t id, const char *name, void *client_data)
           else
             {
               SEXP obj;
-
+              
               if (rank == 1)
                 {
                   if (hdf5_global_verbosity > 2) 
